/*
 * Copyright (C) 2004-2014 L2J DataPack
 * 
 * This file is part of L2J DataPack.
 * 
 * L2J DataPack is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J DataPack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package handlers.admincommandhandlers;

import java.util.StringTokenizer;
import java.util.logging.Logger;

import javolution.lang.MathLib;
import javolution.text.TextBuilder;

import com.l2jserver.Config;
import com.l2jserver.gameserver.datatables.AdminTable;
import com.l2jserver.gameserver.datatables.ExperienceTable;
import com.l2jserver.gameserver.datatables.NpcData;
import com.l2jserver.gameserver.handler.IAdminCommandHandler;
import com.l2jserver.gameserver.idfactory.IdFactory;
import com.l2jserver.gameserver.model.L2World;
import com.l2jserver.gameserver.model.actor.L2Summon;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.model.actor.instance.L2ServitorInstance;
import com.l2jserver.gameserver.model.actor.templates.L2NpcTemplate;
import com.l2jserver.gameserver.model.entity.Hero;
import com.l2jserver.gameserver.model.items.instance.L2ItemInstance;
import com.l2jserver.gameserver.model.olympiad.Olympiad;
import com.l2jserver.gameserver.network.SystemMessageId;
import com.l2jserver.gameserver.network.serverpackets.NpcHtmlMessage;
import com.l2jserver.gameserver.network.serverpackets.SystemMessage;

import handlers.effecthandlers.Summon;

/**
 * This class handles following admin commands: - admin|admin1/admin2/admin3/admin4/admin5 = slots for the 5 starting admin menus - gmliston/gmlistoff = includes/excludes active character from /gmlist results - silence = toggles private messages acceptance mode - diet = toggles weight penalty mode -
 * tradeoff = toggles trade acceptance mode - reload = reloads specified component from multisell|skill|npc|htm|item - set/set_menu/set_mod = alters specified server setting - saveolymp = saves olympiad state manually - manualhero = cycles olympiad and calculate new heroes.
 * @version $Revision: 1.3.2.1.2.4 $ $Date: 2007/07/28 10:06:06 $
 */
public class AdminAdmin implements IAdminCommandHandler
{
	private static final Logger _log = Logger.getLogger(AdminAdmin.class.getName());
	
	private static final String[] ADMIN_COMMANDS =
	{
		"admin_summon",			//[L2J_JP_ADD]
		"admin_unsummon",		//[L2J_JP_ADD]
	//	"admin_bvb",			//[JOJO]
		"admin_item_circle",	//[JOJO]
		"admin_admin",
		"admin_admin1",
		"admin_admin2",
		"admin_admin3",
		"admin_admin4",
		"admin_admin5",
		"admin_admin6",
		"admin_admin7",
		"admin_gmliston",
		"admin_gmlistoff",
		"admin_silence",
		"admin_diet",
		"admin_tradeoff",
		"admin_set",
		"admin_set_mod",
		"admin_saveolymp",
		"admin_sethero",
		"admin_givehero",
		"admin_endolympiad",
		"admin_setconfig",
		"admin_config_server"
	};
	
	@Override
	public boolean useAdminCommand(String command, L2PcInstance activeChar)
	{
		if (command.startsWith("admin_admin"))
		{
			showMainPage(activeChar, command);
		}
		else if (command.equals("admin_config_server"))
		{
			showConfigPage(activeChar);
		}
		else if (command.startsWith("admin_gmliston"))
		{
			AdminTable.getInstance().showGm(activeChar);
			activeChar.sendMessage("Registered into gm list");
			AdminHtml.showAdminHtml(activeChar, "gm_menu.htm");
		}
		else if (command.startsWith("admin_gmlistoff"))
		{
			AdminTable.getInstance().hideGm(activeChar);
			activeChar.sendMessage("Removed from gm list");
			AdminHtml.showAdminHtml(activeChar, "gm_menu.htm");
		}
		else if (command.startsWith("admin_silence"))
		{
			if (activeChar.isSilenceMode()) // already in message refusal mode
			{
				activeChar.setSilenceMode(false);
				activeChar.sendPacket(SystemMessageId.MESSAGE_ACCEPTANCE_MODE);
			}
			else
			{
				activeChar.setSilenceMode(true);
				activeChar.sendPacket(SystemMessageId.MESSAGE_REFUSAL_MODE);
			}
			AdminHtml.showAdminHtml(activeChar, "gm_menu.htm");
		}
		else if (command.startsWith("admin_saveolymp"))
		{
			Olympiad.getInstance().saveOlympiadStatus();
			activeChar.sendMessage("olympiad system saved.");
		}
		else if (command.startsWith("admin_endolympiad"))
		{
			try
			{
				Olympiad.getInstance().manualSelectHeroes();
			}
			catch (Exception e)
			{
				_log.warning("An error occured while ending olympiad: " + e);
			}
			activeChar.sendMessage("Heroes formed.");
		}
		else if (command.startsWith("admin_sethero"))
		{
			if (activeChar.getTarget() == null)
			{
				activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
				return false;
			}
			
			final L2PcInstance target = activeChar.getTarget().isPlayer() ? activeChar.getTarget().getActingPlayer() : activeChar;
			target.setHero(!target.isHero());
			target.broadcastUserInfo();
		}
		else if (command.startsWith("admin_givehero"))
		{
			if (activeChar.getTarget() == null)
			{
				activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
				return false;
			}
			
			final L2PcInstance target = activeChar.getTarget().isPlayer() ? activeChar.getTarget().getActingPlayer() : activeChar;
			if (Hero.getInstance().isHero(target.getObjectId()))
			{
				activeChar.sendMessage("This player has already claimed the hero status.");
				return false;
			}
			else if (Hero.getInstance().canClaim(target.getObjectId()))
			{
				Hero.getInstance().claimHero(target);
			}
			else
			{
				activeChar.sendMessage("This player cannot claim the hero status.");
				return false;
			}
		}
		else if (command.startsWith("admin_diet"))
		{
			try
			{
				StringTokenizer st = new StringTokenizer(command);
				st.nextToken();
				if (st.nextToken().equalsIgnoreCase("on"))
				{
					activeChar.setDietMode(true);
					activeChar.sendMessage("Diet mode on");
				}
				else if (st.nextToken().equalsIgnoreCase("off"))
				{
					activeChar.setDietMode(false);
					activeChar.sendMessage("Diet mode off");
				}
			}
			catch (Exception ex)
			{
				if (activeChar.getDietMode())
				{
					activeChar.setDietMode(false);
					activeChar.sendMessage("Diet mode off");
				}
				else
				{
					activeChar.setDietMode(true);
					activeChar.sendMessage("Diet mode on");
				}
			}
			finally
			{
				activeChar.refreshOverloaded();
			}
			AdminHtml.showAdminHtml(activeChar, "gm_menu.htm");
		}
		else if (command.startsWith("admin_tradeoff"))
		{
			try
			{
				String mode = command.substring(15);
				if (mode.equalsIgnoreCase("on"))
				{
					activeChar.setTradeRefusal(true);
					activeChar.sendMessage("Trade refusal enabled");
				}
				else if (mode.equalsIgnoreCase("off"))
				{
					activeChar.setTradeRefusal(false);
					activeChar.sendMessage("Trade refusal disabled");
				}
			}
			catch (Exception ex)
			{
				if (activeChar.getTradeRefusal())
				{
					activeChar.setTradeRefusal(false);
					activeChar.sendMessage("Trade refusal disabled");
				}
				else
				{
					activeChar.setTradeRefusal(true);
					activeChar.sendMessage("Trade refusal enabled");
				}
			}
			AdminHtml.showAdminHtml(activeChar, "gm_menu.htm");
		}
		else if (command.startsWith("admin_setconfig"))
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();
			try
			{
				String pName = st.nextToken();
				String pValue = st.nextToken();
				if (Config.setParameterValue(pName, pValue))
				{
					activeChar.sendMessage("Config parameter " + pName + " set to " + pValue);
				}
				else
				{
					activeChar.sendMessage("Invalid parameter!");
				}
			}
			catch (Exception e)
			{
				activeChar.sendMessage("Usage: //setconfig <parameter> <value>");
			}
			finally
			{
				showConfigPage(activeChar);
			}
		}
		else if (command.startsWith("admin_set"))
		{
			StringTokenizer st = new StringTokenizer(command);
			String[] cmd = st.nextToken().split("_");
			try
			{
				String[] parameter = st.nextToken().split("=");
				String pName = parameter[0].trim();
				String pValue = parameter[1].trim();
				if (Config.setParameterValue(pName, pValue))
				{
					activeChar.sendMessage("parameter " + pName + " succesfully set to " + pValue);
				}
				else
				{
					activeChar.sendMessage("Invalid parameter!");
				}
			}
			catch (Exception e)
			{
				if (cmd.length == 2)
				{
					activeChar.sendMessage("Usage: //set parameter=value");
				}
			}
			finally
			{
				if (cmd.length == 3)
				{
					if (cmd[2].equalsIgnoreCase("mod"))
					{
						AdminHtml.showAdminHtml(activeChar, "mods_menu.htm");
					}
				}
			}
		}
		
		// [L2J_JP_ADD] GMpT
		else if (command.startsWith("admin_summon"))
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();
			try
			{
				// NPCID̐؂o
				int npcId = Integer.parseInt(st.nextToken());
				if (npcId != 0)
					adminSummon(activeChar, npcId);
			}
			catch (Exception e)
			{
				// NPCID͂ĂȂꍇ̎g\
				activeChar.sendMessage("Usage:  //summon npcid");
			}
		}
		// [L2J_JP_ADD] GMpT()
		else if (command.startsWith("admin_unsummon"))
		{
			// T΁A
			L2Summon s;
			if ((s = activeChar.getSummon()) != null)
				s.unSummon(activeChar);
		}
	//	// [L2J_JP ADD] Boss vs Boss
	//	else if (command.startsWith("admin_bvb"))
	//	{
	//		StringTokenizer st = new StringTokenizer(command);
	//		st.nextToken();
	//		try
	//		{
	//			String cmd = st.nextToken();
	//			if (cmd.equalsIgnoreCase("start"))
	//			{
	//				BvBManager.getInstance().start();
	//			}
	//			else if (cmd.equalsIgnoreCase("end"))
	//			{
	//				BvBManager.getInstance().end();
	//			}
	//			else
	//			{
	//				throw new java.util.NoSuchElementException();
	//			}
	//		}
	//		catch (java.util.NoSuchElementException e)
	//		{
	//			activeChar.sendMessage("Usage://bvb start or //bvb end");
	//		}
	//	}
		else if (command.startsWith("admin_item_circle"))	//[JOJO] VсFAfiŒnʂɁ`
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();
			try {
				int x0 = activeChar.getX();
				int y0 = activeChar.getY();
				int z0 = activeChar.getZ();
				double radius = Integer.parseInt(st.nextToken());
				double count = 0;
				int itemId = 57;
				if (st.hasMoreTokens()) count = Integer.parseInt(st.nextToken());
				if (st.hasMoreTokens()) itemId = Integer.parseInt(st.nextToken());
				int limit = (int)Math.round(MathLib.TWO_PI / 10 * radius);
				if (limit < 8) limit = 8;
				if (limit > 300) limit = 300;
				if (count == 0)
					count = limit;
				else if (count > limit)
				{
					count = limit;
					activeChar.sendMessage("Warning: too many count");
				}
				
				double alpha = MathLib.TWO_PI / count;
				while (--count >= 0)
				{
					double angle = alpha * count;
					int x = x0 + (int) Math.round(Math.cos(angle) * radius);
					int y = y0 + (int) Math.round(Math.sin(angle) * radius);
					int z = z0;
					L2ItemInstance item = new L2ItemInstance(IdFactory.getInstance().getNextId(), itemId);
					item.setOwnerId(activeChar.getObjectId());
					item.new ItemDropTask(item, activeChar, x, y, z).run();
					item.setProtected(false);
					L2World.getInstance().storeObject(item);
				}
			}
			catch (Exception e)
			{
				activeChar.sendMessage("Usage://item_circle radius [count] [item]");
			}
		}

		return true;
	}
	
	@Override
	public String[] getAdminCommandList()
	{
		return ADMIN_COMMANDS;
	}
	
	private void showMainPage(L2PcInstance activeChar, String command)
	{
		int mode = 0;
		String filename = null;
		try
		{
			mode = Integer.parseInt(command.substring(11));
		}
		catch (Exception e)
		{
		}
		switch (mode)
		{
			case 1:
				filename = "main";
				break;
			case 2:
				filename = "game";
				break;
			case 3:
				filename = "effects";
				break;
			case 4:
				filename = "server";
				break;
			case 5:
				filename = "mods";
				break;
			case 6:
				filename = "char";
				break;
			case 7:
				filename = "gm";
				break;
			default:
				filename = "main";
				break;
		}
		AdminHtml.showAdminHtml(activeChar, filename + "_menu.htm");
	}
	
	// [L2J_JP_ADD] GMTs\bh
	public void adminSummon(L2PcInstance activeChar, int npcId)
	{
		// Ƀybgꍇ̓ybg߂B
		L2Summon s;
		if ((s = activeChar.getSummon()) != null)
		{
			SystemMessage sm = SystemMessage.getSystemMessage(SystemMessageId.YOU_ALREADY_HAVE_A_PET);
			activeChar.sendPacket(sm);
			s.unSummon(activeChar);
			s = null;
		}
		
        // (handlers/effecthandlers/Summon.java)̏قڊێʂłB
		//-------------------------------------------------------
		final L2PcInstance player = activeChar.getActingPlayer();
		final L2NpcTemplate template = NpcData.getInstance().getTemplate(npcId);
		L2ServitorInstance summon = new L2ServitorInstance(IdFactory.getInstance().getNextId(), template, player);
		
		summon.setName(template.getName());
		summon.setTitle(activeChar.getName());
	//	summon.setReferenceSkill(???.getSkill().getId());
		summon.setExpMultiplier(1.0f);
		summon.setLifeTime(86400000);
	//	summon.setItemConsume(new ItemHolder(Inventory.ADENA_ID, 1));
	//	summon.setItemConsumeInterval(template.getRace() != NpcRace.SIEGE_WEAPON ? 240000 : 60000);
		
		if (summon.getLevel() >= ExperienceTable.getInstance().getMaxPetLevel())
		{
			summon.getStat().setExp(ExperienceTable.getInstance().getExpForLevel(ExperienceTable.getInstance().getMaxPetLevel() - 1));
			_log.warning(Summon.class.getSimpleName() + ": (" + summon.getName() + ") NpcID: " + summon.getId() + " has a level above " + ExperienceTable.getInstance().getMaxPetLevel() + ". Please rectify.");
		}
		else
		{
			summon.getStat().setExp(ExperienceTable.getInstance().getExpForLevel(summon.getLevel() % ExperienceTable.getInstance().getMaxPetLevel()));
		}
		
		summon.setCurrentHp(summon.getMaxHp());
		summon.setCurrentMp(summon.getMaxMp());
		summon.setHeading(player.getHeading());
		
		player.setPet(summon);
		
		summon.setRunning();
		summon.spawnMe();
		//-------------------------------------------------------
		
		summon.setShowSummonAnimation(false);	// addVisibleObject created the info packets with summon animation
												// if someone comes into range now, the animation shouldnt show any more
		summon.cancelLifeTask();
    }

	public void showConfigPage(L2PcInstance activeChar)
	{
		final NpcHtmlMessage adminReply = new NpcHtmlMessage();
		TextBuilder replyMSG = new TextBuilder("<html><title>L2J :: Config</title><body>"
			+ "<center><table width=270><tr><td width=60><button value=\"Main\" action=\"bypass -h admin_admin\" width=60 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td><td width=150>Config Server Panel</td><td width=60><button value=\"Back\" action=\"bypass -h admin_admin4\" width=60 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr></table></center><br>"
			+ "<center><table width=260><tr><td width=140></td><td width=40></td><td width=40></td></tr>"
			+ "<tr><td><font color=\"00AA00\">Drop:</font></td><td></td><td></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Rate EXP</font> = ").append(Config.RATE_XP).append("</td><td><edit var=\"param1\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig RateXp $param1\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Rate SP</font> = ").append(Config.RATE_SP).append("</td><td><edit var=\"param2\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig RateSp $param2\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Rate Drop Spoil</font> = ").append(Config.RATE_CORPSE_DROP_CHANCE_MULTIPLIER).append("</td><td><edit var=\"param4\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig RateDropSpoil $param4\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td width=140></td><td width=40></td><td width=40></td></tr>"
			+ "<tr><td><font color=\"00AA00\">Enchant:</font></td><td></td><td></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Enchant Element Stone</font> = ").append(Config.ENCHANT_CHANCE_ELEMENT_STONE).append("</td><td><edit var=\"param8\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig EnchantChanceElementStone $param8\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Enchant Element Crystal</font> = ").append(Config.ENCHANT_CHANCE_ELEMENT_CRYSTAL).append("</td><td><edit var=\"param9\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig EnchantChanceElementCrystal $param9\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Enchant Element Jewel</font> = ").append(Config.ENCHANT_CHANCE_ELEMENT_JEWEL).append("</td><td><edit var=\"param10\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig EnchantChanceElementJewel $param10\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
			+ "<tr><td><font color=\"LEVEL\">Enchant Element Energy</font> = ").append(Config.ENCHANT_CHANCE_ELEMENT_ENERGY).append("</td><td><edit var=\"param11\" width=40 height=15></td><td><button value=\"Set\" action=\"bypass -h admin_setconfig EnchantChanceElementEnergy $param11\" width=40 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_ct1.button_df\"></td></tr>"
		
			+ "</table></body></html>");
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}
}
